@@ -67,8 +67,10 @@ void flux_msg_destroy (flux_msg_t *msg);
 
 
 
-int flux_msg_aux_set (const flux_msg_t *msg, const char *name,
-                      void *aux, flux_free_f destroy);
+int flux_msg_aux_set (const flux_msg_t *msg,
+                      const char *name,
+                      void *aux,
+                      flux_free_f destroy);
 void *flux_msg_aux_get (const flux_msg_t *msg, const char *name);
 
 
@@ -88,10 +90,6 @@ int flux_msg_encode (const flux_msg_t *msg, void *buf, size_t size);
 
 
 
-int flux_msg_frames (const flux_msg_t *msg);
-
-
-
 
 
 flux_msg_t *flux_msg_decode (const void *buf, size_t size);
@@ -125,11 +123,6 @@ bool flux_msg_is_noresponse (const flux_msg_t *msg);
 
 
 
-int flux_msg_set_user1 (flux_msg_t *msg);
-bool flux_msg_is_user1 (const flux_msg_t *msg);
-
-
-
 
 int flux_msg_set_topic (flux_msg_t *msg, const char *topic);
 int flux_msg_get_topic (const flux_msg_t *msg, const char **topic);
@@ -146,11 +139,9 @@ bool flux_msg_has_payload (const flux_msg_t *msg);
 
 
 
-
-
-
-int flux_msg_get_flags (const flux_msg_t *msg, uint8_t *flags);
-int flux_msg_set_flags (flux_msg_t *msg, uint8_t flags);
+bool flux_msg_has_flag (const flux_msg_t *msg, int flag);
+int flux_msg_set_flag (flux_msg_t *msg, int flag);
+int flux_msg_clear_flag (flux_msg_t *msg, int flag);
 
 
 
@@ -169,7 +160,7 @@ int flux_msg_vpack (flux_msg_t *msg, const char *fmt, va_list ap);
 
 int flux_msg_unpack (const flux_msg_t *msg, const char *fmt, ...);
 int flux_msg_vunpack (const flux_msg_t *msg, const char *fmt, va_list ap);
-# 294 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 285 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_msg_last_error (const flux_msg_t *msg);
 
 
@@ -261,7 +252,7 @@ void flux_msg_fprint_ts (FILE *f, const flux_msg_t *msg, double timestamp);
 
 
 const char *flux_msg_typestr (int type);
-# 395 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 386 "/__w/flux-python/flux-python/src/_core_clean.h"
 void flux_msg_route_enable (flux_msg_t *msg);
 
 
@@ -320,8 +311,8 @@ char *flux_msg_route_string (const flux_msg_t *msg);
 
 bool flux_msg_route_match_first (const flux_msg_t *msg1,
                                  const flux_msg_t *msg2);
-# 469 "/__w/flux-python/flux-python/src/_core_clean.h"
-typedef struct flux_handle_struct flux_t;
+# 460 "/__w/flux-python/flux-python/src/_core_clean.h"
+typedef struct flux_handle flux_t;
 
 typedef struct {
     int request_tx;
@@ -345,6 +336,7 @@ enum {
     FLUX_O_MATCHDEBUG = 8,
     FLUX_O_TEST_NOSUB = 16,
     FLUX_O_RPCTRACK = 32,
+    FLUX_O_NOREQUEUE = 64,
 };
 
 
@@ -361,7 +353,7 @@ enum {
     FLUX_POLLOUT = 2,
     FLUX_POLLERR = 4,
 };
-# 522 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 515 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_t *flux_open (const char *uri, int flags);
 
 
@@ -397,7 +389,7 @@ int flux_opt_get (flux_t *h, const char *option, void *val, size_t len);
 
 
 void flux_comms_error_set (flux_t *h, flux_comms_error_f fun, void *arg);
-# 566 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 559 "/__w/flux-python/flux-python/src/_core_clean.h"
 void *flux_aux_get (flux_t *h, const char *name);
 int flux_aux_set (flux_t *h, const char *name, void *aux, flux_free_f destroy);
 
@@ -419,7 +411,12 @@ uint32_t flux_matchtag_avail (flux_t *h);
 
 
 int flux_send (flux_t *h, const flux_msg_t *msg, int flags);
-# 595 "/__w/flux-python/flux-python/src/_core_clean.h"
+
+
+
+
+int flux_send_new (flux_t *h, flux_msg_t **msg, int flags);
+# 593 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_msg_t *flux_recv (flux_t *h, struct flux_match match, int flags);
 
 
@@ -434,14 +431,14 @@ int flux_requeue (flux_t *h, const flux_msg_t *msg, int flags);
 
 
 int flux_pollevents (flux_t *h);
-# 619 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 617 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_pollfd (flux_t *h);
 
 
 
 void flux_get_msgcounters (flux_t *h, flux_msgcounters_t *mcs);
 void flux_clr_msgcounters (flux_t *h);
-# 667 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 665 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_buffer flux_buffer_t;
 
 
@@ -458,7 +455,7 @@ int flux_buffer_bytes (flux_buffer_t *fb);
 
 
 int flux_buffer_space (flux_buffer_t *fb);
-# 693 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 691 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_buffer_readonly (flux_buffer_t *fb);
 bool flux_buffer_is_readonly (flux_buffer_t *fb);
 
@@ -466,9 +463,9 @@ bool flux_buffer_is_readonly (flux_buffer_t *fb);
 
 
 int flux_buffer_drop (flux_buffer_t *fb, int len);
-# 708 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 706 "/__w/flux-python/flux-python/src/_core_clean.h"
 const void *flux_buffer_peek (flux_buffer_t *fb, int len, int *lenp);
-# 718 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 716 "/__w/flux-python/flux-python/src/_core_clean.h"
 const void *flux_buffer_read (flux_buffer_t *fb, int len, int *lenp);
 
 
@@ -529,7 +526,7 @@ int flux_buffer_read_to_fd (flux_buffer_t *fb, int fd, int len);
 
 
 int flux_buffer_write_from_fd (flux_buffer_t *fb, int fd, int len);
-# 794 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 792 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_reactor flux_reactor_t;
 
 
@@ -632,7 +629,7 @@ flux_watcher_t *flux_buffer_write_watcher_create (flux_reactor_t *r, int fd,
                                                   int flags, void *arg);
 
 flux_buffer_t *flux_buffer_write_watcher_get_buffer (flux_watcher_t *w);
-# 906 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 904 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_buffer_write_watcher_close (flux_watcher_t *w);
 
 
@@ -710,7 +707,7 @@ struct flux_watcher_ops {
     void (*stop) (flux_watcher_t *w);
     void (*destroy) (flux_watcher_t *w);
 };
-# 991 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 989 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_watcher_t * flux_watcher_create (flux_reactor_t *r, size_t data_size,
                                       struct flux_watcher_ops *ops,
                                       flux_watcher_f fn, void *arg);
@@ -722,7 +719,7 @@ void * flux_watcher_get_data (flux_watcher_t *w);
 
 
 struct flux_watcher_ops * flux_watcher_get_ops (flux_watcher_t *w);
-# 1030 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1028 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_msg_handler flux_msg_handler_t;
 
 typedef void (*flux_msg_handler_f)(flux_t *h, flux_msg_handler_t *mh,
@@ -769,16 +766,20 @@ void flux_msg_handler_delvec (flux_msg_handler_t *msg_handlers[]);
 
 
 int flux_dispatch_requeue (flux_t *h);
-# 1107 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1105 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef flux_t *(connector_init_f)(const char *uri,
                                    int flags,
                                    flux_error_t *errp);
 
 struct flux_handle_ops {
-    int (*setopt)(void *impl, const char *option,
-                          const void *val, size_t len);
-    int (*getopt)(void *impl, const char *option,
-                          void *val, size_t len);
+    int (*setopt)(void *impl,
+                          const char *option,
+                          const void *val,
+                          size_t len);
+    int (*getopt)(void *impl,
+                          const char *option,
+                          void *val,
+                          size_t len);
     int (*pollfd)(void *impl);
     int (*pollevents)(void *impl);
     int (*send)(void *impl, const flux_msg_t *msg, int flags);
@@ -786,11 +787,19 @@ struct flux_handle_ops {
     int (*reconnect)(void *impl);
 
     void (*impl_destroy)(void *impl);
+
+
+    int (*send_new)(void *impl, flux_msg_t **msg, int flags);
+
+
+    void *_pad[4];
 };
 
-flux_t *flux_handle_create (void *impl, const struct flux_handle_ops *ops, int flags);
+flux_t *flux_handle_create (void *impl,
+                            const struct flux_handle_ops *ops,
+                            int flags);
 void flux_handle_destroy (flux_t *hp);
-# 1158 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1168 "/__w/flux-python/flux-python/src/_core_clean.h"
 struct flux_msglist *flux_msglist_create (void);
 void flux_msglist_destroy (struct flux_msglist *l);
 
@@ -804,10 +813,10 @@ const flux_msg_t *flux_msglist_next (struct flux_msglist *l);
 const flux_msg_t *flux_msglist_last (struct flux_msglist *l);
 
 int flux_msglist_count (struct flux_msglist *l);
-# 1179 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1189 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_msglist_pollevents (struct flux_msglist *l);
 int flux_msglist_pollfd (struct flux_msglist *l);
-# 1215 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1225 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_request_decode (const flux_msg_t *msg, const char **topic,
                          const char **s);
 
@@ -838,12 +847,15 @@ flux_msg_t *flux_request_encode (const char *topic, const char *s);
 
 flux_msg_t *flux_request_encode_raw (const char *topic,
                                      const void *data, int len);
-# 1281 "/__w/flux-python/flux-python/src/_core_clean.h"
-int flux_response_decode (const flux_msg_t *msg, const char **topic,
+# 1291 "/__w/flux-python/flux-python/src/_core_clean.h"
+int flux_response_decode (const flux_msg_t *msg,
+                          const char **topic,
                           const char **s);
-# 1292 "/__w/flux-python/flux-python/src/_core_clean.h"
-int flux_response_decode_raw (const flux_msg_t *msg, const char **topic,
-                              const void **data, int *len);
+# 1303 "/__w/flux-python/flux-python/src/_core_clean.h"
+int flux_response_decode_raw (const flux_msg_t *msg,
+                              const char **topic,
+                              const void **data,
+                              int *len);
 
 
 
@@ -859,12 +871,14 @@ flux_msg_t *flux_response_encode (const char *topic, const char *s);
 
 
 flux_msg_t *flux_response_encode_raw (const char *topic,
-                                      const void *data, int len);
+                                      const void *data,
+                                      int len);
 
 
 
 
-flux_msg_t *flux_response_encode_error (const char *topic, int errnum,
+flux_msg_t *flux_response_encode_error (const char *topic,
+                                        int errnum,
                                         const char *errstr);
 
 
@@ -881,26 +895,32 @@ int flux_respond (flux_t *h, const flux_msg_t *request, const char *s);
 
 
 
-int flux_respond_pack (flux_t *h, const flux_msg_t *request,
-                       const char *fmt, ...);
+int flux_respond_pack (flux_t *h,
+                       const flux_msg_t *request,
+                       const char *fmt,
+                       ...);
 
 
 
 
-int flux_respond_raw (flux_t *h, const flux_msg_t *request,
-                      const void *data, int len);
+int flux_respond_raw (flux_t *h,
+                      const flux_msg_t *request,
+                      const void *data,
+                      int len);
 
 
 
 
 
-int flux_respond_error (flux_t *h, const flux_msg_t *request,
-                        int errnum, const char *errstr);
-# 1374 "/__w/flux-python/flux-python/src/_core_clean.h"
+int flux_respond_error (flux_t *h,
+                        const flux_msg_t *request,
+                        int errnum,
+                        const char *errstr);
+# 1395 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_msg_t *flux_control_encode (int type, int status);
 
 int flux_control_decode (const flux_msg_t *msg, int *type, int *status);
-# 1435 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1456 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef void (*flux_log_f)(const char *buf, int len, void *arg);
 
 
@@ -952,7 +972,7 @@ void flux_llog (void *arg,
                 int level,
                 const char *fmt,
                 va_list ap);
-# 1505 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1526 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_future flux_future_t;
 
 typedef void (*flux_continuation_f)(flux_future_t *f, void *arg);
@@ -1018,7 +1038,7 @@ const char * flux_future_first_child (flux_future_t *cf);
 const char * flux_future_next_child (flux_future_t *cf);
 
 flux_future_t *flux_future_get_child (flux_future_t *cf, const char *name);
-# 1583 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1604 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_future_and_then (flux_future_t *f,
                                      flux_continuation_f cb, void *arg);
 
@@ -1046,30 +1066,43 @@ void flux_future_continue_error (flux_future_t *prev, int errnum,
 int flux_future_fulfill_next (flux_future_t *prev,
                               void *result,
                               flux_free_f free_fn);
-# 1625 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1646 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum {
     FLUX_RPC_NORESPONSE = 1,
     FLUX_RPC_STREAMING = 2,
 };
 
-flux_future_t *flux_rpc (flux_t *h, const char *topic, const char *s,
-                         uint32_t nodeid, int flags);
+flux_future_t *flux_rpc (flux_t *h,
+                         const char *topic,
+                         const char *s,
+                         uint32_t nodeid,
+                         int flags);
 
-flux_future_t *flux_rpc_pack (flux_t *h, const char *topic, uint32_t nodeid,
-                              int flags, const char *fmt, ...);
+flux_future_t *flux_rpc_pack (flux_t *h,
+                              const char *topic,
+                              uint32_t nodeid,
+                              int flags,
+                              const char *fmt,
+                              ...);
 
 flux_future_t *flux_rpc_vpack (flux_t *h,
                                const char *topic,
                                uint32_t nodeid,
                                int flags,
-                               const char *fmt, va_list ap);
+                               const char *fmt,
+                               va_list ap);
 
-flux_future_t *flux_rpc_raw (flux_t *h, const char *topic,
-                             const void *data, int len,
-                             uint32_t nodeid, int flags);
+flux_future_t *flux_rpc_raw (flux_t *h,
+                             const char *topic,
+                             const void *data,
+                             int len,
+                             uint32_t nodeid,
+                             int flags);
 
-flux_future_t *flux_rpc_message (flux_t *h, const flux_msg_t *msg,
-                                 uint32_t nodeid, int flags);
+flux_future_t *flux_rpc_message (flux_t *h,
+                                 const flux_msg_t *msg,
+                                 uint32_t nodeid,
+                                 int flags);
 
 int flux_rpc_get (flux_future_t *f, const char **s);
 
@@ -1084,7 +1117,7 @@ uint32_t flux_rpc_get_matchtag (flux_future_t *f);
 
 
 uint32_t flux_rpc_get_nodeid (flux_future_t *f);
-# 1690 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1724 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum event_flags {
     FLUX_EVENT_PRIVATE = 1,
 };
@@ -1165,7 +1198,7 @@ flux_future_t *flux_event_publish_raw (flux_t *h,
 
 
 int flux_event_publish_get_seq (flux_future_t *f, int *seq);
-# 1801 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1835 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum {
     FLUX_MODSTATE_INIT = 0,
     FLUX_MODSTATE_RUNNING = 1,
@@ -1191,7 +1224,7 @@ bool flux_module_debug_test (flux_t *h, int flag, bool clear);
 
 
 int flux_module_set_running (flux_t *h);
-# 1874 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1908 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_attr_get (flux_t *h, const char *name);
 
 
@@ -1230,7 +1263,7 @@ const char *flux_get_hostbyrank (flux_t *h, uint32_t rank);
 
 
 int flux_get_rankbyhost (flux_t *h, const char *host);
-# 1926 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1960 "/__w/flux-python/flux-python/src/_core_clean.h"
 char *flux_hostmap_lookup (flux_t *h,
                            const char *targets,
                            flux_error_t *errp);
@@ -1241,7 +1274,7 @@ char *flux_hostmap_lookup (flux_t *h,
 
 
 int flux_get_instance_starttime (flux_t *h, double *starttime);
-# 1964 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 1998 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum flux_conf_flags {
     FLUX_CONF_INSTALLED=0,
     FLUX_CONF_INTREE=1,
@@ -1294,13 +1327,13 @@ int flux_conf_vunpack (const flux_conf_t *conf,
 int flux_conf_unpack (const flux_conf_t *conf,
                       flux_error_t *error,
                       const char *fmt, ...);
-# 2048 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2082 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_barrier (flux_t *h, const char *name, int nprocs);
-# 2091 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2125 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_service_register (flux_t *h, const char *name);
-# 2103 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2137 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_service_unregister (flux_t *h, const char *name);
-# 2155 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2189 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_core_version_string (void);
 
 
@@ -1309,7 +1342,7 @@ const char *flux_core_version_string (void);
 
 
 int flux_core_version (int *major, int *minor, int *patch);
-# 2193 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2227 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum {
     FLUX_PLUGIN_RTLD_LAZY = 1,
     FLUX_PLUGIN_RTLD_NOW = 2,
@@ -1387,7 +1420,7 @@ flux_plugin_f flux_plugin_match_handler (flux_plugin_t *p, const char *topic);
 int flux_plugin_register (flux_plugin_t *p,
                           const char *name,
                           const struct flux_plugin_handler t[]);
-# 2280 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2314 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_plugin_aux_set (flux_plugin_t *p,
                          const char *key,
                          void *val,
@@ -1449,14 +1482,14 @@ int flux_plugin_arg_unpack (flux_plugin_arg_t *args, int flags,
                             const char *fmt, ...);
 int flux_plugin_arg_vunpack (flux_plugin_arg_t *args, int flags,
                              const char *fmt, va_list ap);
-# 2349 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2383 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_plugin_call (flux_plugin_t *p, const char *name,
                       flux_plugin_arg_t *args);
-# 2359 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2393 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_plugin_load_dso (flux_plugin_t *p, const char *path);
-# 2392 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2426 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_sync_create (flux_t *h, double minimum);
-# 2424 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2458 "/__w/flux-python/flux-python/src/_core_clean.h"
 bool flux_disconnect_match (const flux_msg_t *msg1, const flux_msg_t *msg2);
 
 
@@ -1476,7 +1509,7 @@ bool flux_cancel_match (const flux_msg_t *msg1, const flux_msg_t *msg2);
 int flux_msglist_cancel (flux_t *h,
                          struct flux_msglist *l,
                          const flux_msg_t *msg);
-# 2493 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2527 "/__w/flux-python/flux-python/src/_core_clean.h"
 void flux_stats_count (flux_t *h, const char *name, ssize_t count);
 
 
@@ -1512,12 +1545,14 @@ void flux_stats_set_prefix (flux_t *h, const char *fmt, ...);
 
 
 bool flux_stats_enabled (flux_t *h, const char *metric);
-# 2576 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2610 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_kvsdir flux_kvsdir_t;
 typedef struct flux_kvsitr flux_kvsitr_t;
-# 2591 "/__w/flux-python/flux-python/src/_core_clean.h"
-flux_kvsdir_t *flux_kvsdir_create (flux_t *handle, const char *rootref,
-                                   const char *key, const char *json_str);
+# 2625 "/__w/flux-python/flux-python/src/_core_clean.h"
+flux_kvsdir_t *flux_kvsdir_create (flux_t *handle,
+                                   const char *rootref,
+                                   const char *key,
+                                   const char *json_str);
 void flux_kvsdir_destroy (flux_kvsdir_t *dir);
 
 flux_kvsdir_t *flux_kvsdir_copy (const flux_kvsdir_t *dir);
@@ -1561,10 +1596,14 @@ const char *flux_kvsdir_rootref (const flux_kvsdir_t *dir);
 
 
 char *flux_kvsdir_key_at (const flux_kvsdir_t *dir, const char *key);
-# 2664 "/__w/flux-python/flux-python/src/_core_clean.h"
-flux_future_t *flux_kvs_lookup (flux_t *h, const char *ns, int flags,
+# 2700 "/__w/flux-python/flux-python/src/_core_clean.h"
+flux_future_t *flux_kvs_lookup (flux_t *h,
+                                const char *ns,
+                                int flags,
                                 const char *key);
-flux_future_t *flux_kvs_lookupat (flux_t *h, int flags, const char *key,
+flux_future_t *flux_kvs_lookupat (flux_t *h,
+                                  int flags,
+                                  const char *key,
                                   const char *treeobj);
 
 int flux_kvs_lookup_get (flux_future_t *f, const char **value);
@@ -1584,76 +1623,102 @@ const char *flux_kvs_lookup_get_key (flux_future_t *f);
 
 
 int flux_kvs_lookup_cancel (flux_future_t *f);
-# 2714 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2754 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_getroot (flux_t *h, const char *ns, int flags);
-# 2723 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2763 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_kvs_getroot_get_treeobj (flux_future_t *f, const char **treeobj);
 int flux_kvs_getroot_get_blobref (flux_future_t *f, const char **blobref);
 int flux_kvs_getroot_get_sequence (flux_future_t *f, int *seq);
 int flux_kvs_getroot_get_owner (flux_future_t *f, uint32_t *owner);
-# 2755 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2795 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_kvs_txn flux_kvs_txn_t;
 
 flux_kvs_txn_t *flux_kvs_txn_create (void);
 void flux_kvs_txn_destroy (flux_kvs_txn_t *txn);
 
-int flux_kvs_txn_put (flux_kvs_txn_t *txn, int flags,
-                      const char *key, const char *value);
+int flux_kvs_txn_put (flux_kvs_txn_t *txn,
+                      int flags,
+                      const char *key,
+                      const char *value);
 
-int flux_kvs_txn_vpack (flux_kvs_txn_t *txn, int flags, const char *key,
-                        const char *fmt, va_list ap);
+int flux_kvs_txn_vpack (flux_kvs_txn_t *txn,
+                        int flags,
+                        const char *key,
+                        const char *fmt,
+                        va_list ap);
 
-int flux_kvs_txn_pack (flux_kvs_txn_t *txn, int flags, const char *key,
-                       const char *fmt, ...);
+int flux_kvs_txn_pack (flux_kvs_txn_t *txn,
+                       int flags,
+                       const char *key,
+                       const char *fmt,
+                       ...);
 
-int flux_kvs_txn_put_raw (flux_kvs_txn_t *txn, int flags,
-                          const char *key, const void *data, int len);
+int flux_kvs_txn_put_raw (flux_kvs_txn_t *txn,
+                          int flags,
+                          const char *key,
+                          const void *data,
+                          int len);
 
-int flux_kvs_txn_put_treeobj (flux_kvs_txn_t *txn, int flags,
-                              const char *key, const char *treeobj);
+int flux_kvs_txn_put_treeobj (flux_kvs_txn_t *txn,
+                              int flags,
+                              const char *key,
+                              const char *treeobj);
 
-int flux_kvs_txn_mkdir (flux_kvs_txn_t *txn, int flags,
+int flux_kvs_txn_mkdir (flux_kvs_txn_t *txn,
+                        int flags,
                         const char *key);
 
-int flux_kvs_txn_unlink (flux_kvs_txn_t *txn, int flags,
+int flux_kvs_txn_unlink (flux_kvs_txn_t *txn,
+                         int flags,
                          const char *key);
 
-int flux_kvs_txn_symlink (flux_kvs_txn_t *txn, int flags,
-                          const char *key, const char *ns,
+int flux_kvs_txn_symlink (flux_kvs_txn_t *txn,
+                          int flags,
+                          const char *key,
+                          const char *ns,
                           const char *target);
-# 2833 "/__w/flux-python/flux-python/src/_core_clean.h"
+
+int flux_kvs_txn_clear (flux_kvs_txn_t *txn);
+
+bool flux_kvs_txn_is_empty (flux_kvs_txn_t *txn);
+# 2894 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum kvs_commit_flags {
     FLUX_KVS_NO_MERGE = 1,
     FLUX_KVS_TXN_COMPACT = 2,
     FLUX_KVS_SYNC = 4,
 };
 
-flux_future_t *flux_kvs_commit (flux_t *h, const char *ns, int flags,
+flux_future_t *flux_kvs_commit (flux_t *h,
+                                const char *ns,
+                                int flags,
                                 flux_kvs_txn_t *txn);
 
-flux_future_t *flux_kvs_fence (flux_t *h, const char *ns, int flags,
-                               const char *name, int nprocs,
+flux_future_t *flux_kvs_fence (flux_t *h,
+                               const char *ns,
+                               int flags,
+                               const char *name,
+                               int nprocs,
                                flux_kvs_txn_t *txn);
 
 
 int flux_kvs_commit_get_treeobj (flux_future_t *f, const char **treeobj);
 int flux_kvs_commit_get_rootref (flux_future_t *f, const char **rootref);
 int flux_kvs_commit_get_sequence (flux_future_t *f, int *rootseq);
-# 2889 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2955 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_copy (flux_t *h,
                               const char *srcns,
                               const char *srckey,
                               const char *dstns,
                               const char *dstkey,
                               int commit_flags);
-# 2906 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2972 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_future_t *flux_kvs_move (flux_t *h,
                               const char *srcns,
                               const char *srckey,
                               const char *dstns,
                               const char *dstkey,
                               int commit_flags);
-# 2929 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 2995 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum kvs_op {
     FLUX_KVS_READDIR = 1,
     FLUX_KVS_READLINK = 2,
@@ -1665,12 +1730,16 @@ enum kvs_op {
     FLUX_KVS_WATCH_UNIQ = 128,
     FLUX_KVS_WATCH_APPEND = 256
 };
-# 2950 "/__w/flux-python/flux-python/src/_core_clean.h"
-flux_future_t *flux_kvs_namespace_create (flux_t *h, const char *ns,
-                                          uint32_t owner, int flags);
-flux_future_t *flux_kvs_namespace_create_with (flux_t *h, const char *ns,
+# 3016 "/__w/flux-python/flux-python/src/_core_clean.h"
+flux_future_t *flux_kvs_namespace_create (flux_t *h,
+                                          const char *ns,
+                                          uint32_t owner,
+                                          int flags);
+flux_future_t *flux_kvs_namespace_create_with (flux_t *h,
+                                               const char *ns,
                                                const char *rootref,
-                                               uint32_t owner, int flags);
+                                               uint32_t owner,
+                                               int flags);
 flux_future_t *flux_kvs_namespace_remove (flux_t *h, const char *ns);
 
 
@@ -1685,7 +1754,7 @@ int flux_kvs_wait_version (flux_t *h, const char *ns, int version);
 
 
 int flux_kvs_dropcache (flux_t *h);
-# 2997 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3067 "/__w/flux-python/flux-python/src/_core_clean.h"
 enum job_submit_flags {
     FLUX_JOB_PRE_SIGNED = 1,
     FLUX_JOB_DEBUG = 2,
@@ -1751,10 +1820,10 @@ typedef uint64_t flux_jobid_t;
 
 
 int flux_job_id_parse (const char *s, flux_jobid_t *id);
-# 3071 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3141 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_job_id_encode (flux_jobid_t id, const char *type,
                         char *buf, size_t bufsz);
-# 3082 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3152 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_job_statetostr (flux_job_state_t state, const char *fmt);
 
 
@@ -1868,16 +1937,16 @@ flux_future_t *flux_job_result (flux_t *h, flux_jobid_t id, int flags);
 
 int flux_job_result_get (flux_future_t *f,
                          const char **json_str);
-# 3213 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3283 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_job_result_get_unpack (flux_future_t *f, const char *fmt, ...);
-# 3227 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3297 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_job_timeleft (flux_t *h, flux_error_t *errp, double *timeleft);
-# 3243 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3313 "/__w/flux-python/flux-python/src/_core_clean.h"
 char *flux_unwrap_string (const char *in,
                           bool verify,
                           uint32_t *userid,
                           flux_error_t *error);
-# 3271 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3341 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_jobspec1 flux_jobspec1_t;
 typedef flux_error_t flux_jobspec1_error_t;
 
@@ -1975,7 +2044,7 @@ char *flux_jobspec1_encode (flux_jobspec1_t *jobspec, size_t flags);
 
 flux_jobspec1_t *flux_jobspec1_decode (const char *s,
                                        flux_jobspec1_error_t *error);
-# 3382 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3452 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_jobspec1_t *flux_jobspec1_from_command (int argc,
                                              char **argv,
                                              char **env,
@@ -1987,7 +2056,7 @@ flux_jobspec1_t *flux_jobspec1_from_command (int argc,
 
 
 void flux_jobspec1_destroy (flux_jobspec1_t *jobspec);
-# 3424 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3494 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef struct flux_command flux_cmd_t;
 
 
@@ -1997,7 +2066,7 @@ typedef struct flux_command flux_cmd_t;
 
 
 typedef struct flux_subprocess flux_subprocess_t;
-# 3444 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3514 "/__w/flux-python/flux-python/src/_core_clean.h"
 typedef enum {
     FLUX_SUBPROCESS_INIT,
     FLUX_SUBPROCESS_RUNNING,
@@ -2070,9 +2139,9 @@ typedef void (*subprocess_log_f) (void *arg,
                                   int level,
                                   const char *fmt,
                                   va_list args);
-# 3528 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3598 "/__w/flux-python/flux-python/src/_core_clean.h"
 void flux_standard_output (flux_subprocess_t *p, const char *stream);
-# 3537 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3607 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_cmd_t * flux_cmd_create (int argc, char *argv[], char **env);
 
 
@@ -2147,12 +2216,12 @@ const char *flux_cmd_getenv (const flux_cmd_t *cmd, const char *name);
 
 int flux_cmd_setcwd (flux_cmd_t *cmd, const char *cwd);
 const char *flux_cmd_getcwd (const flux_cmd_t *cmd);
-# 3624 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3694 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_cmd_add_channel (flux_cmd_t *cmd, const char *name);
-# 3674 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3747 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_cmd_setopt (flux_cmd_t *cmd, const char *var, const char *val);
 const char *flux_cmd_getopt (flux_cmd_t *cmd, const char *var);
-# 3697 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3770 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_subprocess_t *flux_local_exec (flux_reactor_t *r,
                                     int flags,
                                     const flux_cmd_t *cmd,
@@ -2178,17 +2247,11 @@ flux_subprocess_t *flux_rexec_ex (flux_t *h,
                                   const flux_subprocess_ops_t *ops,
                                   subprocess_log_f log_fn,
                                   void *log_data);
-# 3732 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3805 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_subprocess_stream_start (flux_subprocess_t *p, const char *stream);
 int flux_subprocess_stream_stop (flux_subprocess_t *p, const char *stream);
 int flux_subprocess_stream_status (flux_subprocess_t *p, const char *stream);
-
-
-
-
-
-
-
+# 3818 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_subprocess_write (flux_subprocess_t *p, const char *stream,
                            const char *buf, size_t len);
 
@@ -2198,11 +2261,11 @@ int flux_subprocess_write (flux_subprocess_t *p, const char *stream,
 
 
 int flux_subprocess_close (flux_subprocess_t *p, const char *stream);
-# 3763 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3839 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_subprocess_read (flux_subprocess_t *p,
                                   const char *stream,
                                   int len, int *lenp);
-# 3778 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3854 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_subprocess_read_line (flux_subprocess_t *p,
                                        const char *stream,
                                        int *lenp);
@@ -2213,10 +2276,10 @@ const char *flux_subprocess_read_line (flux_subprocess_t *p,
 const char *flux_subprocess_read_trimmed_line (flux_subprocess_t *p,
                                                const char *stream,
                                                int *lenp);
-# 3798 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3874 "/__w/flux-python/flux-python/src/_core_clean.h"
 int flux_subprocess_read_stream_closed (flux_subprocess_t *p,
                                         const char *stream);
-# 3817 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3893 "/__w/flux-python/flux-python/src/_core_clean.h"
 const char *flux_subprocess_getline (flux_subprocess_t *p,
                                      const char *stream,
                                      int *lenp);
@@ -2236,7 +2299,7 @@ flux_future_t *flux_subprocess_kill (flux_subprocess_t *p, int signo);
 
 void flux_subprocess_ref (flux_subprocess_t *p);
 void flux_subprocess_unref (flux_subprocess_t *p);
-# 3844 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3920 "/__w/flux-python/flux-python/src/_core_clean.h"
 flux_subprocess_state_t flux_subprocess_state (flux_subprocess_t *p);
 
 
@@ -2298,7 +2361,7 @@ void *flux_subprocess_aux_get (flux_subprocess_t *p, const char *name);
 
 void flux_subprocess_channel_incref (flux_subprocess_t *p, const char *name);
 void flux_subprocess_channel_decref (flux_subprocess_t *p, const char *name);
-# 3918 "/__w/flux-python/flux-python/src/_core_clean.h"
+# 3994 "/__w/flux-python/flux-python/src/_core_clean.h"
 extern "Python" void message_handler_wrapper(flux_t *, flux_msg_handler_t *, const flux_msg_t *, void *);
 
 
